#
#    Forecasting with 2-d smooth models
#
#    rm(list = ls())
#
#    Get some data
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_UK_Deaths_2013.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_UK_Exposures_2013.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#    Choose male data for ages 50 to 90 and years 1961 to 2013
AGE = 50:90
YEAR = 1961:2013 
Dth = Death.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Exp = Exposure.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Obs = log(Dth/Exp)
n.x = nrow(Dth)
n.y = ncol(Dth)
#  Install MortalitySmooth package
library("svcm", lib.loc = "/home/iain/reml/R.LIBRARIES")
library("MortalitySmooth", lib.loc = "/home/iain/reml/R.LIBRARIES")
#
#      Forecasting
#
Fit.2d = Mort2Dsmooth(AGE, YEAR, Dth, offset=log(Exp), over = TRUE)
c(Fit.2d$lambda, Fit.2d$df, Fit.2d$dev, Fit.2d$psi2)
YEAR.NEW = 1961:2038
NEW.DATA = list(x = AGE, y = YEAR.NEW)
Forecast.2d = predict(Fit.2d, newdata = NEW.DATA)
Central = Forecast.2d
# 
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
Age.Plot = c(60,75,90)  # Set plotting age
for(i in 1:3){
  Row = Age.Plot[i] - min(AGE) + 1
  plot(YEAR+0.5, Obs[Row, ], xlim = range(YEAR.NEW),
                         ylim = range(Obs[Row,], Central[Row,]), xlab = "Year",
       ylab = expression(paste("log(",mu,")")), cex = 0.5, pch = 16)
  lines(YEAR.NEW + 0.5, Central[Row, ])
  legend("bottomleft", paste("Age =", Age.Plot[i]), bty = "n")
}
# 
plot(AGE+0.5, Forecast.2d[ , 53], type = "l", xlab = "Age",
     ylab = expression(paste("log(",mu,")")),
     ylim = range(Forecast.2d[ , 53],  Forecast.2d[ , 78]), lwd = 2)
lines(AGE+0.5, Forecast.2d[ , 78], lty = 2, lwd = 2)
legend("topleft", legend = c("Year = 2013", "Year = 2038"), lty = c(1,2), lwd = 2,
       bty = "n")
#
#      Now force lambdas
#
Fit.2d = Mort2Dsmooth(AGE, YEAR, Dth, offset=log(Exp),
  method = 3, lambdas = c(2000, 4000))
c(Fit.2d$lambda, Fit.2d$df, Fit.2d$dev,Fit.2d$psi2)
YEAR.NEW = 1961:2038
NEW.DATA = list(x = AGE, y = YEAR.NEW)
Forecast.2d = predict(Fit.2d, newdata = NEW.DATA, se.fit = TRUE)
Forecast.Pspline = Forecast.2d$fit
Central = Forecast.2d$fit; SE = sqrt(Fit.2d$psi2) * Forecast.2d$se.fit
CI.UP = Central + 1.96 * SE; CI.DN = Central - 1.96 * SE
#
#    Graphical output
#
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
Age.Plot = c(60,75,90)  # Set plotting age
for(i in 1:3){
  Row = Age.Plot[i] - min(AGE) + 1
  plot(YEAR, Obs[Row, ], xlim = range(YEAR.NEW),
                         ylim = range(Obs[Row,], CI.DN[Row,]), xlab = "Year",
       ylab = expression(paste("log(",mu,")")), cex = 0.5, pch = 16)
  lines(YEAR.NEW, Central[Row, ])
  lines(YEAR.NEW, CI.UP[Row, ], lty = 2)
  lines(YEAR.NEW, CI.DN[Row, ], lty = 2)
  legend("bottomleft", paste("Age =", Age.Plot[i]), bty = "n")
}
# 
plot(AGE, Forecast.2d$fit[ , 53], type = "l", xlab = "Age",
     ylab = expression(paste("log(",mu,")")),
     ylim = range(Forecast.2d$fit[ , 53],  Forecast.2d$fit[ , 78]), lwd = 2)
lines(AGE, Forecast.2d$fit[ , 78], lty = 2, lwd = 2)
legend("topleft", legend = c("Year = 2013", "Year = 2038"), lty = c(1,2), lwd = 2,
       bty = "n")



