#
#     Plots a B-spline basis
#
#     Load function to obtain B-spline regression matrix
#
#     rm(list = ls())    #  Optionally empty workspace
#
source("Bspline.r")
#     Load library splines
library(splines)
#
#     Plot basis functions over years 1961-2009
#
ndx = 6      #  Gap between knots = (2009 - 1961)/6 = 8 years
bdeg = 3
X = seq(1961, 2009, by = 0.1)   #  Tiny increment gives a smooth plot
Setup = bspline(X, min(X), max(X), ndx, bdeg)
names(Setup)
B.X = Setup$B     #   Extract the regression matrix
dim(B.X)          #   There are 9 B-splines in the basis
Setup$Knots       #   Note some external knots are required
# 
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(X, B.X[,6], type = "n", xlab = "Year", ylab = "Bspline", axes = FALSE,
     xlim = c(1960, 2010), lwd = 2)
for(j in 1:ncol(B.X)) lines(X, B.X[ ,j], lwd = 2)
axis(1, at = seq(1950, 2010, by = 10), tcl = -0.4)
axis(2, at = seq(-0.1,0.7, 0.1), tcl = -0.4)
Plot.Knots = Setup$Knots[4:(length(Setup$Knots) - 3)]
points(Plot.Knots, rep(0, length(Plot.Knots)), pch = 16, cex = 1.4)
# 
