#
#     Function to compute B-spline regression matrix
#
#     x = vector of x-values
#     xl = min(x), xr = max(x)
#     ndx = number of intervals in domain
#           ie, number of internal knots = ndx - 1
#     bdeg = degree of B-spline (quadratic = 2, cubic = 3, etc)
#
#     NB: Requires the R-library splines
#
bspline <- function(x, xl, xr, ndx, bdeg){
  dx <- (xr-xl)/ndx
  knots <- seq(xl-bdeg*dx, xr+bdeg*dx, by=dx)
  B <- spline.des(knots,x,bdeg+1,0*x)$design
  return(list(B = B, Knots = knots))
}
