#
#      Fit CBD models
#
#      rm(list = ls())
#
#      Get some data
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_UK_Deaths_2013.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_UK_Exposures_2013.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#    Choose male data for ages 50 to 90 and years 1961 to 2013
AGE = 50:90
YEAR = 1961:2013 
Dth = Death.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Exp = Exposure.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Obs = log(Dth/Exp)
n.x = nrow(Dth)
n.y = ncol(Dth)
#    Convert data to vector form
Dth.V = c(Dth); Exp.V = c(Exp)
#    Obtain model matrix for CBD model
X1 = kronecker(diag(n.y), rep(1, n.x))
Age.mean = AGE - mean(AGE)
X2 = kronecker(diag(n.y), Age.mean)
X = cbind(X1, X2)
#    Fit model
CBD.Male = glm(Dth.V ~ -1 + X, offset = log(Exp.V), family = poisson)
Coef = CBD.Male$coef
Kappa.1 = Coef[1:n.y]     #  First n.y values
Kappa.2 = Coef[-(1:n.y)]  #  Drop first n.y values
CBD.Male$dev    #    Deviance
#    Fitted log(mortality) from (a) fitted deaths (b) X kappa
Fitted.M.hat = matrix(log(CBD.Male$fit/Exp.V), n.x, n.y)
Fitted.M.Hat = matrix(X %*% CBD.Male$coef, n.x, n.y)
max(abs(Fitted.M.hat - Fitted.M.Hat))     #  Equal of course
#
#    Graphical output
#
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
plot(YEAR, Kappa.1, xlab = "Year", ylab = expression(kappa[1]))
#
plot(YEAR, Kappa.2, xlab = "Year", ylab = expression(kappa[2])) 
#
plot(YEAR+0.5, log(Dth/Exp)[16, ], xlab = "Year",
     ylab = expression(paste("log(",mu,")")))
lines(YEAR+0.5, Fitted.M.hat[16, ])
legend("topright", legend = c("Data", "CBD"), pch = c(1, -1), lty = c(-1, 1),
       bty = "n")
legend("bottomleft", legend = "Age 65", bty = "n")
#
plot(AGE+0.5, log(Dth/Exp)[ , 40], xlab = "Age",
     ylab = expression(paste("log(",mu,")")))
lines(AGE+0.5, Fitted.M.hat[ , 40])
legend("topleft", legend = c("Data", "CBD"), pch = c(1, -1), lty = c(-1, 1),
       bty = "n")
legend("bottomright", legend = "Year 2000", bty = "n")






