#
#    Forecasting with the CBD model
#
#    rm(list = ls())
#
#    Get some data
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_UK_Deaths_2013.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_UK_Exposures_2013.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#    Choose male data for ages 50 to 90 and years 1961 to 2013
AGE = 50:90
YEAR = 1961:2013 
Dth = Death.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Exp = Exposure.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Obs = log(Dth/Exp)
n.x = nrow(Dth)
n.y = ncol(Dth)
#    Convert data to vector form
Dth.V = c(Dth); Exp.V = c(Exp)
#
#    Fit CBD model as before
#
#    Obtain model matrix for CBD model
#
X1 = kronecker(diag(n.y), rep(1, n.x))
Age.mean = AGE - mean(AGE)
X2 = kronecker(diag(n.y), Age.mean)
X = cbind(X1, X2)
#    Fit model
CBD.Male = glm(Dth.V ~ -1 + X, offset = log(Exp.V), family = poisson)
Coef = CBD.Male$coef
Kappa.1 = Coef[1:n.y]     #  First n.y values
Kappa.2 = Coef[-(1:n.y)]  #  Drop first n.y values
CBD.Male$dev    #    Deviance
#    Fitted log(mortality) from (a) fitted deaths (b) X kappa
Fitted.M.hat = matrix(log(CBD.Male$fit/Exp.V), n.x, n.y)
Fitted.M.Hat = matrix(X %*% CBD.Male$coef, n.x, n.y)
max(abs(Fitted.M.hat - Fitted.M.Hat))     #  Equal of course
#
#     Parameter estimation
#
Diff = cbind(diff(Kappa.1), diff(Kappa.2))
Mu = apply(Diff, 2, mean)
#    Or
Mu.2 = c(Kappa.1[n.y] - Kappa.1[1], Kappa.2[n.y] - Kappa.2[1])/(n.y-1)
max(abs(Mu - Mu.2))
#
S2 = apply(Diff, 2, var)
Cov = cov(Diff[ ,1], Diff[ ,2])
Sigma = matrix(c(S2[1], Cov, Cov, S2[2]), 2, 2)
Rho = Cov/sqrt(Sigma[1,1]*Sigma[2,2])
#    Or
Sigma.2 = matrix(0, 2, 2)
Sigma.2[1, 1] = sum((diff(Kappa.1) - Mu[1])^2)/(n.y - 2)
Sigma.2[2, 1] = sum((diff(Kappa.1) - Mu[1])*(diff(Kappa.2) -
         Mu[2]))/(n.y - 2)
Sigma.2[1, 2] = Sigma.2[2, 1]
Sigma.2[2, 2] = sum((diff(Kappa.2) - Mu[2])^2)/(n.y - 2)
max(abs(Sigma - Sigma.2))
#
#     Forecasting - follows text closely
#
n.f = 25
I.f = diag(n.f)
X1.f = kronecker(I.f, rep(1, n.x))
X2.f = kronecker(I.f, Age.mean)
X.f = cbind(X1.f, X2.f)
c.f = 1:n.f
Kappa.f = c(Kappa.1[n.y], Kappa.2[n.y]) + Mu %*% t(c.f)
Kappa.f = c(t(Kappa.f))
Forecast = X.f %*% Kappa.f
dim(Forecast) = c(n.x, n.f)
Var.Mat.Forecast = 1/(n.y -1) * X.f %*% kronecker(Sigma, c.f %*% t(c.f)) %*% t(X.f)
Var.Forecast = diag(Var.Mat.Forecast)
dim(Var.Forecast) = c(n.x, n.f)
SD.Forecast = sqrt(Var.Forecast)
SD.Forecast.CBD = SD.Forecast
#
Forecast.Years = (YEAR[n.y]+1):(YEAR[n.y]+n.f)
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
plot(YEAR, Kappa.1, xlab = "Year", ylab = expression(kappa[1]),
     xlim = range(YEAR, Forecast.Years), ylim = range(Kappa.1,Kappa.f[1:n.f]),
     cex = 1)
lines(Forecast.Years, Kappa.f[1:n.f], pch = 16, cex = 1)
plot(YEAR, Kappa.2, xlab = "Year", ylab = expression(kappa[2]),
     xlim = range(YEAR, Forecast.Years), ylim = range(Kappa.2,Kappa.f[-(1:n.f)]),
     cex = 1)
lines(Forecast.Years, Kappa.f[-(1:n.f)], pch = 16, cex = 1)
Plot.Age = c(60, 80)
for(i in 1:2){
  Plot.Row = Plot.Age[i] - min(AGE) + 1
  plot(YEAR + 0.5, Obs[Plot.Row, ], xlim = c(YEAR[1], YEAR[n.y] + n.f),
       ylim = range(Obs[Plot.Row, ], Forecast[Plot.Row, ]),
       xlab = "Year", ylab = expression(paste("log(",mu,")")))
  SD = SD.Forecast[Plot.Row, ]
  lines(Forecast.Years + 0.5, Forecast[Plot.Row, ])
  lines(Forecast.Years + 0.5, Forecast[Plot.Row, ] + 1.96 * SD)
  lines(Forecast.Years + 0.5, Forecast[Plot.Row, ] - 1.96 * SD)
  legend("bottomleft", legend = paste("Age =", Plot.Age[i]), bty = "n")
}
