#
#    Fit Lee-Carter model
#
#  rm(list = ls())
#
#    Get some data
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_UK_Deaths_2013.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_UK_Exposures_2013.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose male data for ages 50 to 90 and years 1961 to 2013
#
AGE = 50:90
YEAR = 1961:2013 
Dth = Death.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Exp = Exposure.Data$Male.Matrix[(Age > 49) & (Age < 91), Year > 1960]
Obs = log(Dth/Exp)
n.x = nrow(Dth)
n.y = ncol(Dth)
#
persp(AGE+0.5, YEAR, Obs, theta = -45, phi = 30, ticktype = "detailed",
      xlab = "Age", ylab = "Year", zlab = "log(mortality)",
      cex.lab = 0.8, cex.axis = 0.8, col = "lightgrey")
#
#     MLE with gnm( )
#
library("gnm", lib.loc = "/home/iain/reml/R.LIBRARIES")
Age.F = factor(rep(AGE, n.y))          #  Make age a factor
Year.F = factor(rep(YEAR, each = n.x)) #  Make year a factor
Dth.V = c(Dth)   #  Make Dth a vector
Exp.V = c(Exp)   #  Make exp a vector
LC.Male = gnm(Dth.V ~ -1 + Age.F + Mult(Age.F, Year.F),
                offset = log(Exp.V), family = poisson)
#
#     Get coefficients
#
Alpha.gnm = LC.Male$coef[1:n.x]
Beta.gnm = LC.Male$coef[(n.x+1):(2*n.x)]
Kappa.gnm = LC.Male$coef[(2*n.x+1):(2*n.x+n.y)]
#
#    Convert to standard Lee-Carter parameters and check 
#      1) location and scale constraints
#      2) invariance of fitted log mortality
#
Kappa.m = mean(Kappa.gnm)
Beta.m = mean(Beta.gnm)
Alpha.hat = Alpha.gnm + Kappa.m*Beta.gnm
Beta.hat = Beta.gnm/(n.x * Beta.m)
Kappa.hat = n.x * Beta.m*(Kappa.gnm - Kappa.m)
Fitted.M.hat = Alpha.hat + Beta.hat %*% t(Kappa.hat)
#
c(sum(Kappa.hat), sum(Beta.hat))                     # 1) OK
max(abs(c(Fitted.M.hat) - log(LC.Male$fit/Exp.V)))   # 2) OK
#
#    Graphical output
#
par(mfrow = c(2,2), mar = c(4.5,4.5,1,1))
plot(AGE, Alpha.hat, xlab = "Age", ylab = expression(alpha), cex = 0.5, pch = 16)
#
plot(AGE, Beta.hat, xlab = "Age", ylab = expression(beta), cex = 0.5, pch = 16)
#
plot(YEAR, Kappa.hat, xlab = "Year", ylab = expression(kappa), cex = 0.5, pch = 16)
#  
Age.Plot = 65  # Set plotting age
Row = Age.Plot - min(AGE) + 1
plot(YEAR + 0.5, Fitted.M.hat[Row, ], xlab = "Year", ylab =
     expression(paste("log(",mu,")")), type = "l")
points(YEAR + 0.5, Obs[Row, ], cex = 0.5, pch = 16)
legend("topright", legend = c("Data", "LC"), pch = c(16, -1), lty = c(-1, 1),
       bty = "n")
legend("bottomleft", legend = paste("Age", Age.Plot), bty = "n")


