#
#    P-spline methods
#
#    rm(list = ls())
#
#    Get some data
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_USA_Deaths.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_USA_Exposures.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose female data for age 70 for years 1961 to 2009
#
YEAR = 1961:2009 
Dth = Death.Data$Female.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Exp = Exposure.Data$Female.Matrix[Age == 70, (Year > 1960) &
  (Year < 2010)]
#
Obs = log(Dth/Exp)
#
library("svcm", lib.loc = "/home/iain/reml/R.LIBRARIES")
library("MortalitySmooth", lib.loc = "/home/iain/reml/R.LIBRARIES")
#
#     Find ED, Dev, Aic & Bic for various lambda
#                                       
LAMBDA = c(0, 10^2, 10^4, 10^6, 10^8)
ED = NULL; DEV = NULL; AIC = NULL; BIC = NULL
Fit = function(Lambda) Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp),
  ndx = 17, lambda = Lambda, method = 3)
for(i in 1:length(LAMBDA)) {
  FIT = Fit(LAMBDA[i])
  ED = c(ED, FIT$df); DEV = c(DEV, FIT$dev)
  AIC = c(AIC, FIT$aic); BIC = c(BIC, FIT$bic)
}
cbind(LAMBDA, ED, DEV, AIC, BIC)
#
#     Optimal AIC and BIC
#
LAMBDA = 1:200
AIC = NULL
for(i in 1:length(LAMBDA)) {
  FIT = Fit(LAMBDA[i])
  AIC = c(AIC, FIT$aic)
}
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(LAMBDA, AIC, xlab = expression(lambda), type = "l", lwd = 2,
     xlim = c(0, 200), ylim = c(228, 231), axes = FALSE)
axis(1,las = 1, at = seq(-10, 210, by = 20), tcl = -0.4)
axis(2, seq(227.5, 231, by = 0.5), tcl = -0.4)
text(140, 228.2, expression(paste("Optimal ", lambda, " = 43.7, AIC = 228.34")),
     cex = 1.2)
#
Opt.aic = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp), ndx = 17,
  method = 2, control = list(TOL2 = 0.01))
summary(Opt.aic)   #   Estimated psi^2 = 6.18
#
Opt.bic = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp), ndx = 17,
  method = 1, control = list(TOL2 = 0.01))
summary(Opt.bic)   #   Estimated psi^2 = 6.07
#
#  We need to deal with the overdisperion so we set over = TRUE in the call
#
Opt.aic.over = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp), ndx = 17,
  method = 2, over = TRUE, control = list(TOL2 = 0.01))
summary(Opt.aic.over)
#
Opt.bic.over = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp), ndx = 17,
  method = 1, over = TRUE, control = list(TOL2 = 0.01))
summary(Opt.bic.over)
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR, Obs, xlab = "Year", xlim = c(1960, 2010), ylim = c(-4.15, -3.5),
     ylab = "log(mortality)", axes = FALSE)
axis(1,las = 1, at = seq(1950, 2010, by = 10), tcl = -0.4)
axis(2, seq(-4.2,-3.5, by = 0.1), tcl = -0.4)
lines(YEAR, Opt.bic.over$logm, lwd = 2)
lines(YEAR, Opt.aic.over$logm, lwd = 2, lty = 2)
legend(1960, -4, legend = c("Observed mortality",
       "P-spline fit with BIC and overdispersion",
       "P-spline fit with AIC and overdispersion"),
       lty = c(-1, 1, 2), lwd = c(-1, 2, 2), pch = c(1, -1, -1), bty = "n")    


