#
#    Forecasting with P-splines: 1-d
#
#    Get some data
#
#    rm(list = ls())
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_USA_Deaths.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_USA_Exposures.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose female data for age 70 for years 1961 to 2009
#
YEAR = 1961:2009 
Dth = Death.Data$Female.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Exp = Exposure.Data$Female.Matrix[Age == 70, (Year > 1960) &
  (Year < 2010)]
#
Obs = log(Dth/Exp)
#
source("Bspline.r")    #  Load function to compute regression matrix
library(splines)       #  Load spline library
#      Load packages
library("svcm", lib.loc = "/home/iain/reml/R.LIBRARIES")
library("MortalitySmooth", lib.loc = "/home/iain/reml/R.LIBRARIES")
#
#      Forecasting for 25 years
#      First we use the dummy data method
#
Weight = c(rep(1, length(YEAR)), rep(0,25))
YEAR.NEW = YEAR[1]:(YEAR[length(YEAR)] + 25)
Dth.NEW = c(Dth, rep(1, 25))
Exp.NEW = c(Exp, rep(2, 25))
Smooth.Forecast = Mort1Dsmooth(x = YEAR.NEW, y = Dth.NEW, offset = log(Exp.NEW),
  w = Weight, over = TRUE)
#
#    Check linearity of forecast coefficients
#
Range = max(YEAR.NEW) - min(YEAR.NEW)
Xmax = max(YEAR.NEW) + 0.01 * Range   #  Technical: look at Mort1Dsmooth code
Xmin = min(YEAR.NEW) - 0.01 * Range   #                  "
ndx = Smooth.Forecast$ndx
Delta = (Xmax - Xmin)/ndx
Knots = seq(Xmin, Xmax, by = Delta)
Coef = Smooth.Forecast$coef
cbind(c(0,Knots,0), Coef, c(0, diff(Coef)))
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR, Obs, xlab = "Year", xlim = c(1960, 2035), ylim = c(-4.8, -3.5),
     ylab = "log(mortality)", axes = FALSE)
axis(1,las = 1, at = seq(1950, 2040, by = 10), tcl = -0.4)
axis(2, seq(-5.2,-3.5, by = 0.1), tcl = -0.4)
lines(YEAR.NEW, Smooth.Forecast$logm, lwd = 2)
points(Knots, Coef[2:(length(Coef)-1)], pch = 15)
legend(1960, -4.2, legend = c("Observed mortality", "P-spline regression",
                            "P-spline coefficients"),
       lty = c(-1, 1, -1), lwd = c(-1, 2, -1),
       pch = c(1, -1, 15), bty = "n")
#
#     Forecasting with the predict function
#
Smooth.Fit = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp), over = TRUE)
Smooth.Forecast = predict(Smooth.Fit, newdata = YEAR.NEW, se.fit = TRUE)
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR, Obs, xlab = "Year", xlim = c(1960, 2035), ylim = c(-4.9, -3.4),
     ylab = "log(mortality)", axes = FALSE)
axis(1,las = 1, at = seq(1950, 2040, by = 10), tcl = -0.4)
axis(2, seq(-5.0,-3.4, by = 0.2), tcl = -0.4)
lines(YEAR.NEW, Smooth.Forecast$fit, lwd = 2)
lines(YEAR.NEW, Smooth.Forecast$fit + 1.96 * Smooth.Forecast$se.fit,
      lty = 2, lwd = 2)
lines(YEAR.NEW, Smooth.Forecast$fit - 1.96 * Smooth.Forecast$se.fit,
      lty = 2, lwd = 2)
legend(1960, -4.2, legend = c("Observed mortality", "Fit and forecast",
                            "95% CI"),
       lty = c(-1, 1, 2), lwd = c(-1, 2, 2),
       pch = c(1, -1, -1), bty = "n")
