#
#    Fits P-spline regression model
#    US female data, age 70, years 1961 to 2009
#
#    Get some data
#
#    rm(list = ls())    # Optionally empty workspace
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_USA_Deaths.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_USA_Exposures.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose female data for age 70 for years 1961 to 2009
#
YEAR = 1961:2009 
Dth = Death.Data$Female.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Exp = Exposure.Data$Female.Matrix[Age == 70, (Year > 1960) &
  (Year < 2010)]
#
Obs = log(Dth/Exp)
#
#  ====================================================
#
#   Load MortalitySmooth library.  This requires the library svcm
#
library("svcm", lib.loc = "/home/iain/reml/R.LIBRARIES")
library("MortalitySmooth", lib.loc = "/home/iain/reml/R.LIBRARIES")
#
#  Plot: Data + P-spline regression fit + coefficients
#
#  Force lambda = 10000 with method = 3; see help(Mort1Dsmooth)
#
Smooth.Fit = Mort1Dsmooth(x = YEAR, y = Dth, offset = log(Exp),
  ndx = 17, lambda = 10000, method = 3)
#  Calculate knot positions: note this requires detailed inspection
#  of the Mort1Dsmooth code
Range = max(YEAR) - min(YEAR)
Xmax = max(YEAR) + 0.01 * Range
Xmin = min(YEAR) - 0.01 * Range
ndx = Smooth.Fit$ndx
Delta <- (Xmax - Xmin)/ndx
Knots = seq(Xmin - Delta, Xmax + Delta, by = Delta)
cbind(Knots, Smooth.Fit$coef)
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR, Obs, xlab = "Year", xlim = c(1960, 2010), ylim = c(-4.15, -3.5),
     ylab = "log(mortality)", axes = FALSE)
axis(1,las = 1, at = seq(1950, 2010, by = 10), tcl = -0.4)
axis(2, seq(-4.2,-3.5, by = 0.1), tcl = -0.4)
lines(YEAR, Smooth.Fit$logm, lwd = 2)
points(Knots, Smooth.Fit$coef, pch = 15)
legend(1960, -3.9, legend = c("Observed mortality", "P-spline regression",
                            "P-spline coefficients"),
       lty = c(-1, 1, -1), lwd = c(-1, 2, -1),
       pch = c(1, -1, 15), bty = "n")
#
#
#  Plot: Data + P-spline regression fit + coefficients
#
#  Choose optimal lambda by minimizing BIC; see help(Mort1Dsmooth)
#
Smooth.Fit = Mort1Dsmooth(YEAR, Dth, offset = log(Exp), ndx = 17, method = 1)
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR, Obs, xlab = "Year", xlim = c(1960, 2010), ylim = c(-4.15, -3.5),
     ylab = "log(mortality)", axes = FALSE)
axis(1,las = 1, at = seq(1950, 2010, by = 10), tcl = -0.4)
axis(2, seq(-4.2,-3.5, by = 0.1), tcl = -0.4)
#    Force lambda = 10000 with method = 3; see help(Mort1Dsmooth)
lines(YEAR, Smooth.Fit$logm, lwd = 2)
points(Knots, Smooth.Fit$coef, pch = 15)
legend(1960, -3.9, legend = c("Observed mortality", "P-spline regression",
                            "P-spline coefficients"),
       lty = c(-1, 1, -1), lwd = c(-1, 2, -1),
       pch = c(1, -1, 15), bty = "n")
#
