#
#   File name: Read_HMD.r
#
#   Function to read in data from Human Mortality Database.
#
#   Reads the file of deaths (or exposures) downloaded from HMD and
#   creates male and female death (or exposure) matrices for
#   ages 0-109 and the range of years available
#
#   Function name: Read.HMD
#
#   Auxiliary function: check.integer
#
#   Argument: file    : name of HMD file, as in "File.Name"
#
#   Output: Female.Matrix, Male.Matrix
#                matrices of deaths (or exposures),
#                rows indexed by ages 0-109, columns by available years
#           Age: 0-109 (always)
#           Year: available years
#
#   Auxiliary function to check if a number is an integer
#
check.integer <- function(x) {
    x == round(x)
}
#  
Read.HMD <- function(file){
  x <- scan(file, skip = 3, what = "character") 
  x <- matrix(x, ncol = 5, byrow = T)
  #
  #   Delete the rows for ages 110+
  #
  Select <- rep(1, nrow(x))
  Select[check.integer((1:nrow(x))/111)] <- 0
  x <- x[Select == 1, ]
  x <- matrix(as.numeric(c(x)), ncol = 5)
  #
  #   Find years and select female and male data
  #
  Age <- 0:109
  Year <- min(x[ ,1]) : max(x[ ,1])
  Female <- x[ ,3]
  Female.Matrix <- matrix(Female, nrow = 110)
  dimnames(Female.Matrix) <- list(Age, Year)
  Male <- x[ ,4]
  Male.Matrix <- matrix(Male, nrow = 110)
  dimnames(Male.Matrix) <- list(Age, Year)
  return(list(Age = Age, Year = Year, Female.Matrix = Female.Matrix,
              Male.Matrix = Male.Matrix))
}



