#
#    Fits quadratic and cubic models of mortality
#
#    Get some data
#
#    rm(list = ls())
#
source("Read_HMD.r")
#
Death.Data = Read.HMD("Data/HMD_USA_Deaths.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_USA_Exposures.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose male and female data for age 70 for year 1961 to 2009
#
YEAR = 1961:2009
Dth.Male = Death.Data$Male.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Dth.Female = Death.Data$Female.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Exp.Male = Exposure.Data$Male.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
Exp.Female = Exposure.Data$Female.Matrix[Age == 70, (Year > 1960) & (Year < 2010)]
#
Obs.Male = log(Dth.Male/Exp.Male)
Obs.Female = log(Dth.Female/Exp.Female)
#
#    Plot the data
#
YEAR.Plot = YEAR + 1/2
Y.lim = range(Obs.Male, Obs.Female)
plot(YEAR.Plot, Obs.Male, ylim = Y.lim, xlab = "Year", ylab = "log(mortality)")
points(YEAR.Plot, Obs.Female, pch = 2)
legend("bottomleft", legend = c("Male", "Female"), pch = c(1,2), bty="n")
#
#    Fit quadratic and cubic curves
#
x = YEAR - mean(YEAR); x2 = x^2; x3 = x^3
#
Fit.Male.Quad = glm(Dth.Male ~ x + x2 + offset(log(Exp.Male)), family = poisson)
names(Fit.Male.Quad)
lines(YEAR, log(Fit.Male.Quad$fit/Exp.Male))
#
Fit.Female.Cube = glm(Dth.Female ~ x + x2 + x3 + offset(log(Exp.Female)),
  family = poisson)
lines(YEAR, log(Fit.Female.Cube$fit/Exp.Female), lty = 2)
#
#    Complete plot
#
par(mar=c(4.2, 4, 0, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
plot(YEAR.Plot, Obs.Male, ylim = Y.lim, xlab = "Year", ylab = "log(mortality)",
     axes = FALSE)
axis(1, las = 1, at = seq(1950,2010,10)); axis(2, las = 1, at = seq(-4.2,-2.8,0.2))
points(YEAR.Plot, Obs.Female, pch = 2)
lines(YEAR.Plot, log(Fit.Male.Quad$fit/Exp.Male), lwd = 2)
lines(YEAR.Plot, log(Fit.Female.Cube$fit/Exp.Female), lty = 2, lwd = 2)
legend("bottomleft", legend = c("Male", "Female", "Quadratic", "Cubic"),
       pch = c(1,2,-1,-1), lty = c(-1,-1,1,2), , lwd = c(-1,-1,2,2), bty="n")
#
#    With model matrix
#
X = cbind(x, x2, x3)
Fit.Female.Cube.2 = glm(Dth.Female ~ X + offset(log(Exp.Female)),
  family = poisson)
max(abs(Fit.Female.Cube$coef -Fit.Female.Cube.2$coef)) 
#
