#
#    Whittaker smoothing
#
#    Load the HMD read function Read_HMD from the file Read_HMD.r
#
#    rm(list = ls())   #   Optionally empty workspace
#
source("Read_HMD.r")
#
#    Get some data from the directory Data
#
Death.Data = Read.HMD("Data/HMD_UK_Deaths.txt")
names(Death.Data)
Exposure.Data = Read.HMD("Data/HMD_UK_Exposures.txt")
names(Exposure.Data)
#
Age = Death.Data$Age; Age
Year = Death.Data$Year; Year
#
#    Choose male data for ages 2-30 for year 2000
#
AGE = 2:30
Dth = Death.Data$Male.Matrix[((min(AGE) - 1) < Age) & (Age < (max(AGE) + 1)),
  Year == 2000]
Exp = Exposure.Data$Male.Matrix[((min(AGE) - 1) < Age) & (Age < (max(AGE) + 1)),
  Year == 2000]
cbind(AGE, Dth, Exp)
#
#    Load function to perform Whittaker
#
source("Whittaker_Smooth.r")
Ans = WhittakerSmooth(AGE, Dth, Exp, 10)     #   lambda = 10
names(Ans)
cbind(AGE, Ans$Obs, Ans$Smooth)
#
par(mar=c(4.2, 4, 2, 0.5), mgp=c(3, 1, 0), las=1, cex=1)
AGE.Plot = AGE + 1/2
plot(AGE.Plot, Ans$Obs, type = "n", xlab = "Age", 
     ylab = expression(paste("log(", mu,")")), axes = FALSE,
     main = "Whittaker smooth of UK male mortality in 2000")
axis(1, las = 1, at = seq(0,35,5)); axis(2, las = 1, at = seq(-10,-6, 0.5))
points(AGE.Plot, Ans$Obs)
lines(AGE.Plot, Ans$Smooth, col = "red", lwd = 2)
legend("bottomright", pch=c(1, -1), lty = c(-1,1),
       legend=c("Observed", "Smooth"), bty="n")
