#
#   File name: Whittaker_Smooth.r
#
#   Function to perform Whittaker smoothing
#
#   Fit a Whittaker smooth for given ages, deaths, central
#   exposures and smoothing parameter.
#
#   Function name: WhittakerSmooth
#
#       Input: a = ages
#              d = deaths
#              e = central exposures
#              l = smoothing parameter, lambda
#
#       Output: Obs = observed log(mortalities)
#               Smooth = smoothed log(mortalities)
#
WhittakerSmooth = function(a, d, e, l){
  Obs = log(d/e) 
  n = length(a)
  Id = diag(n)
  D3 = diff(Id, diff = 3)
  Smooth = solve(Id + l*t(D3) %*% D3, Obs)
  return(list(Obs = Obs, Smooth = Smooth))
}




 
